/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.yifu.cloud.plus.v1.yifu.salary.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.TableField;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttribute;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttributeConstants;
import com.yifu.cloud.plus.v1.yifu.salary.entity.SysMessageSalary;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * 最低工资提醒-每月更新一次
 *
 * @author fxj
 * @date 2022-08-05 11:40:14
 */
@Data
public class SysMessageSalaryExportVo extends SysMessageSalary {
	/**
	 * 客户名称
	 */
	@ExcelAttribute(name = "客户名称", isNotEmpty = true, errorInfo = "客户名称不能为空", maxLength = 50)
	@NotBlank(message = "客户名称不能为空")
	@Length(max = 50, message = "客户名称不能超过50个字符")
	@ExcelProperty("客户名称")
	private String customerName;
	/**
	 * 部门名称
	 */
	@ExcelAttribute(name = "部门名称", isNotEmpty = true, errorInfo = "部门名称不能为空", maxLength = 50)
	@NotBlank(message = "部门名称不能为空")
	@Length(max = 50, message = "部门名称不能超过50个字符")
	@ExcelProperty("部门名称")
	private String departName;
	/**
	 * 员工姓名
	 */
	@ExcelAttribute(name = "员工姓名", maxLength = 50)
	@Length(max = 50, message = "employeeName不能超过50个字符")
	@ExcelProperty("员工姓名")
	private String employeeName;
	/**
	 * employeeIdnum
	 */
	@ExcelAttribute(name = "employeeIdnum", maxLength = 50)
	@Length(max = 50, message = "employeeIdnum不能超过50个字符")
	@ExcelProperty("employeeIdnum")
	private String employeeIdnum;
	/**
	 * 工资月
	 */
	@ExcelAttribute(name = "工资月", maxLength = 10)
	@Length(max = 10, message = "工资月不能超过10个字符")
	@ExcelProperty("工资月")
	private String salaryMonth;

	/**
	 * 实发工资
	 */
	@ExcelAttribute(name = "实发工资", isNotEmpty = true, errorInfo = "实发工资不能为空")
	@NotBlank(message = "实发工资不能为空")
	@ExcelProperty("实发工资")
	private BigDecimal actualSalary;
	/**
	 * 应发工资
	 */
	@ExcelAttribute(name = "应发工资")
	@ExcelProperty("应发工资")
	private BigDecimal relaySalary;
	/**
	 * 省
	 */
	@ExcelAttribute(name = "省")
	@ExcelProperty("省")
	private Integer province;
	/**
	 * 市
	 */
	@ExcelAttribute(name = "市")
	@ExcelProperty("市")
	private Integer city;
	/**
	 * 县
	 */
	@ExcelAttribute(name = "县")
	@ExcelProperty("县")
	private Integer town;
	/**
	 * 最低工资标准
	 */
	@ExcelAttribute(name = "最低工资标准")
	@ExcelProperty("最低工资标准")
	private BigDecimal salaryBase;

	/**
	 * 工资制表人-创建人
	 */
	@ExcelAttribute(name = "工资制表人-创建人", maxLength = 32)
	@Length(max = 32, message = "工资制表人-创建人不能超过32个字符")
	@ExcelProperty("创建人")
	private String salaryMakeMan;

	/**
	 * 反馈
	 */
	@ExcelAttribute(name = "反馈", maxLength = 300)
	@Length(max = 300, message = "反馈不能超过300个字符")
	@ExcelProperty("反馈")
	private String feedback;

	/**
	 * 原因大类 根据数据字典来判断
	 */
	@TableField(exist = false)
	@Schema(description = "原因大类 根据数据字典来判断 ", name = "reasonType")
	@ExcelAttribute(name = "原因大类", errorInfo = "原因大类不能为空", maxLength = 32, isDataId = true, dataType = ExcelAttributeConstants.HAVE_SALARY_NO_SOCIAL_TYPE, needExport = true)
	private Integer reasonType;

}
